function [X,ns,xs,ys] = RandPatchesFromImagesCell(MiniBatchSize,PatchSize,Images,ns,xs,ys)
%% code from Zoran & Weiss (2012)
% return a set of MiniBatchSize patches sized PatchSizexPatchSize from the
% Images cell
% Slightly edited by Sander Keemink
% Inputs:
%   - MiniBatchSize: number of patches
%   - PatchSize:     size of patch. A patch is PatchSize x PatchSize. 
%   - Images:        Cell containing image set
%   - ns,xs,ys:      ignore
% 
%  [X] = RandPatchesFromImagesCell(MiniBatchSize,PatchSize,Images)
% 
[N] = length(Images);

X = zeros(PatchSize,PatchSize,MiniBatchSize);
shouldSample = false;
if ~exist('ns','var')
    ns = randsample(N,MiniBatchSize,true);
    xs = [];
    ys = [];
    shouldSample = true;
end
for n=sort(unique(ns)')
    inds = find(ns==n);
    [H,W] = size(Images{n});
    if shouldSample
        ys(inds) = randsample(H-PatchSize+1,length(inds),true);
        xs(inds) = randsample(W-PatchSize+1,length(inds),true);
    end
    image_inds = sub2ind([H W],ys(inds),xs(inds));
    for y=0:PatchSize-1
        for x=0:PatchSize-1
            X(y+1,x+1,inds) = Images{n}(image_inds + y + x*H);
        end
    end
end

X = reshape(X,[PatchSize^2 MiniBatchSize]);

% Subtract local mean gray-scale value from each patch in X to give output Y
% DC = single(ones(size(X,1),1)*mean(X));
% Y = single(X-DC);
% DC = mean(X,1);
% X = bsxfun(@minus,X,DC);
% Remove DC component
X=removeDC(X);

% remove flat patches
inds = find(std(X,1)<0.002);
X(:,inds) = [];

% if strcmp(class(Images{1}),'uint8')
%     X = single(X/255);
% end